<?php
/**
 * @package Blox Page Builder
 * @author UniteCMS.net
 * @copyright (C) 2017 Unite CMS, All Rights Reserved. 
 * @license GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * */
defined('_JEXEC') or die('Restricted access');


class HelperProviderUC{

	private static $isTitleSet = false;
	private static $arrFilters = array();
	private static $mergedFilters = array();
	
	
	/**
	 * set toolbar title
	 */
	public static function setToolbarTitle($titleName = null){
		
		if(self::$isTitleSet == true)
			return(false);
		
		self::$isTitleSet = true;
		
		$pluginName = JText::_('com_addonlibrary');
		
		if(empty($titleName))
			$showTitle = $pluginName;
		else
			$showTitle = $pluginName." - ".$titleName;
		
		JToolBarHelper::title($showTitle, 'generic.png');
		
	}
	
	
	/**
	 * Check value to find if it was serialized.
	 *
	 */
	public static function is_serialized( $data, $strict = true ) {
		// if it isn't a string, it isn't serialized.
		if ( ! is_string( $data ) ) {
			return false;
		}
		$data = trim( $data );
		if ( 'N;' == $data ) {
			return true;
		}
		if ( strlen( $data ) < 4 ) {
			return false;
		}
		if ( ':' !== $data[1] ) {
			return false;
		}
		if ( $strict ) {
			$lastc = substr( $data, -1 );
			if ( ';' !== $lastc && '}' !== $lastc ) {
				return false;
			}
		} else {
			$semicolon = strpos( $data, ';' );
			$brace     = strpos( $data, '}' );
			// Either ; or } must exist.
			if ( false === $semicolon && false === $brace )
				return false;
			// But neither must be in the first X characters.
			if ( false !== $semicolon && $semicolon < 3 )
				return false;
			if ( false !== $brace && $brace < 4 )
				return false;
		}
		$token = $data[0];
		switch ( $token ) {
			case 's' :
				if ( $strict ) {
					if ( '"' !== substr( $data, -2, 1 ) ) {
						return false;
					}
				} elseif ( false === strpos( $data, '"' ) ) {
					return false;
				}
				// or else fall through
			case 'a' :
			case 'O' :
				return (bool) preg_match( "/^{$token}:[0-9]+:/s", $data );
			case 'b' :
			case 'i' :
			case 'd' :
				$end = $strict ? '$' : '';
				return (bool) preg_match( "/^{$token}:[0-9.E-]+;$end/", $data );
		}
		return false;
	}
	
	
	/**
	 * Unserialize value only if it was serialized.
	 *
	 */
	public static function maybe_unserialize( $original ) {
		if ( self::is_serialized( $original ) ) // don't attempt to unserialize data that wasn't serialized going in
			return @unserialize( $original );
		return $original;
	}
	
	
	/**
	 * check if ssl by php methods
	 */
	public static function isSSLByPHP() {
		
	    if ( isset($_SERVER['HTTPS']) ) {
	        if ( 'on' == strtolower($_SERVER['HTTPS']) )
	            return true;
	        if ( '1' == $_SERVER['HTTPS'] )
	            return true;
	    } elseif ( isset($_SERVER['SERVER_PORT']) && ( '443' == $_SERVER['SERVER_PORT'] ) ) {
	        return true;
	    }
	    return false;
	}	
	
	
	private function a____________SANITIZE_TEXT_FIELD____________(){}
	
	
	/**
	 * 
	 * check invalida utf-8
	 */
	public static function check_invalid_utf8( $string, $strip = false ) {
		$string = (string) $string;
	
		if ( 0 === strlen( $string ) ) {
			return '';
		}
	
		// Store the site charset as a static to avoid multiple calls to get_option()
		$is_utf8 = true;
	
		// Check for support for utf8 in the installed PCRE library once and store the result in a static
		static $utf8_pcre = null;
		if ( ! isset( $utf8_pcre ) ) {
			$utf8_pcre = @preg_match( '/^./u', 'a' );
		}
		// We can't demand utf8 in the PCRE installation, so just return the string in those cases
		if ( !$utf8_pcre ) {
			return $string;
		}
	
		// preg_match fails when it encounters invalid UTF8 in $string
		if ( 1 === @preg_match( '/^./us', $string ) ) {
			return $string;
		}
	
		// Attempt to strip the bad chars if requested (not recommended)
		if ( $strip && function_exists( 'iconv' ) ) {
			return iconv( 'utf-8', 'utf-8', $string );
		}
	
		return '';
	}
	
	
	/**
	 * process less trhen
	 */
	public static function pre_kses_less_than_callback( $matches ) {
		if ( false === strpos($matches[0], '>') )
			return esc_html($matches[0]);
		return $matches[0];
	}
	
	
	/**
	 * process less then
	 */
	public static function pre_kses_less_than( $text ) {
		return preg_replace_callback('%<[^>]*?((?=<)|>|$)%', array('HelperProviderUC', 'pre_kses_less_than_callback'), $text);
	}

	/**
	 * strip all tags
	 */
	public static function strip_all_tags($string, $remove_breaks = false) {
		$string = preg_replace( '@<(script|style)[^>]*?>.*?</\\1>@si', '', $string );
		$string = strip_tags($string);
	
		if ( $remove_breaks )
			$string = preg_replace('/[\r\n\t ]+/', ' ', $string);
	
		return trim( $string );
	}
	
	
	/**
	 * sanitize text fields
	 */
	public static function sanitizeTextField( $str, $keep_newlines = false ) {
		
		$filtered = self::check_invalid_utf8( $str );

		if ( strpos($filtered, '<') !== false ) {
			$filtered = self::pre_kses_less_than( $filtered );
			// This will strip extra whitespace for us.
			$filtered = self::strip_all_tags( $filtered, false );
	
			// Use html entities in a special case to make sure no later
			// newline stripping stage could lead to a functional tag
			$filtered = str_replace("<\n", "&lt;\n", $filtered);
		}
	
		if ( ! $keep_newlines ) {
			$filtered = preg_replace( '/[\r\n\t ]+/', ' ', $filtered );
		}
		$filtered = trim( $filtered );
	
		$found = false;
		while ( preg_match('/%[a-f0-9]{2}/i', $filtered, $match) ) {
			$filtered = str_replace($match[0], '', $filtered);
			$found = true;
		}
	
		if ( $found ) {
			// Strip out the whitespace that may now exist after removing the octets.
			$filtered = trim( preg_replace('/ +/', ' ', $filtered) );
		}
	
		return $filtered;
	}
	
	private function a____________SANITIZE____________(){}
	
	/**
	 * sanitize key
	 */
	public static function sanitizeKey($key){
		$raw_key = $key;
		$key = strtolower( $key );
		$key = preg_replace( '/[^a-z0-9_\-]/', '', $key );
		return($key);
	}
	
	
	
	private function a____________FITLERS____________(){}
	
	
	/**
	 * Hook a function or method to a specific filter action.
	 *
	 */
	public static function add_filter( $tag, $function_to_add, $priority = 10, $accepted_args = 1 ) {
		
		$idx = self::filter_build_unique_id($tag, $function_to_add, $priority);
		
		self::$arrFilters[$tag][$priority][$idx] = array('function' => $function_to_add, 'accepted_args' => $accepted_args);
		
		
		return true;
	}
	
	
	/**
	 * apply filters
	 */
	public static function apply_filters( $tag, $value ) {
		
		$args = array();
	
		if(isset(self::$arrFilters[$tag]) == false)
			return($value);
	
		// Sort.
		if ( !isset( self::$mergedFilters[ $tag ] ) && isset(self::$arrFilters[$tag]) ) {
			ksort(self::$arrFilters[$tag]);
			self::$mergedFilters[ $tag ] = true;
		}
	
	
		reset( self::$arrFilters[$tag] );
	
		if ( empty($args) )
			$args = func_get_args();
	
		do {
			foreach( (array) current(self::$arrFilters[$tag]) as $the_ )
				if ( !is_null($the_['function']) ){
				$args[1] = $value;
	
				$value = call_user_func_array($the_['function'], array_slice($args, 1, (int) $the_['accepted_args']));
			}
	
		} while ( next(self::$arrFilters[$tag]) !== false );
	
	
		return $value;
	}
	
	
	/**
	 * do action
	 */
	public static function do_action($tag) {
	
		$args = array();
	
		if(isset(self::$arrFilters[$tag]) == false)
			return(false);
	
		// Sort.
		if ( !isset( self::$mergedFilters[ $tag ] ) && isset(self::$arrFilters[$tag]) ) {
			ksort(self::$arrFilters[$tag]);
			self::$mergedFilters[ $tag ] = true;
		}
	
		reset( self::$arrFilters[$tag] );
	
		if ( empty($args) )
			$args = func_get_args();
	
		do {
			foreach( (array) current(self::$arrFilters[$tag]) as $the_ )
				if ( !is_null($the_['function']) ){
				call_user_func_array($the_['function'], array_slice($args, 1, (int) $the_['accepted_args']));
			}
	
		} while ( next(self::$arrFilters[$tag]) !== false );
	
	
	}
	
	
	/**
	 * build unique ID
	 */
	private static function filter_build_unique_id($tag, $function, $priority) {
		global $wp_filter;
		static $filter_id_count = 0;
		
		if ( is_string($function) )
			return $function;
		
		if ( is_object($function) ) {
			// Closures are currently implemented as objects
			$function = array( $function, '' );
		} else {
			$function = (array) $function;
		}
				
		if (is_object($function[0]) ) {
			// Object Class Calling
			if ( function_exists('spl_object_hash') ) {
				return spl_object_hash($function[0]) . $function[1];
			} else {
				$obj_idx = get_class($function[0]).$function[1];
				if ( !isset($function[0]->wp_filter_id) ) {
					if ( false === $priority )
						return false;
					$obj_idx .= isset($wp_filter[$tag][$priority]) ? count((array)$wp_filter[$tag][$priority]) : $filter_id_count;
					$function[0]->wp_filter_id = $filter_id_count;
					++$filter_id_count;
				} else {
					$obj_idx .= $function[0]->wp_filter_id;
				}
	
				return $obj_idx;
			}
		} elseif ( is_string( $function[0] ) ) {
			// Static Calling
			return $function[0] . '::' . $function[1];
		}
		
		
	}
	
	private function a____________GLOBAL_INIT____________(){}
	
	
	/**
	 * modify provider general settings
	 */
	private static function modifyGeneralSettingsProvider(UniteSettingsUC $objSettings){
		
		
		/**
		 * filter general settings - joomla
		 */
		
		//hide joomla only settings
		
		if(UniteProviderFunctionsUC::$isStandAlone == false){
			
			$objSettings->updateSettingProperty("form_admin_mail_from", "hidden", true);		
			$objSettings->updateSettingProperty("form_admin_mail_from_name", "hidden", true);		
			
			return($objSettings);
		}
		
		//hide remove elfinder option
		
		$arrUpdateHidden = array("joomla_media_selector");
			
		$objSettings->updateSettingProperty("joomla_media_selector", "hidden", true);		
		
		//modify home page selector
		$objLayouts = new UniteCreatorLayouts();
		$arrLayouts = $objLayouts->getArrLayoutsShort(true);
		$arrLayouts = UniteFunctionsUC::arrayToAssoc($arrLayouts,"title","id");
		
		$objSettings->updateSettingItems("home_page_id", $arrLayouts, "empty");
		
		
		return($objSettings);
	}
	
	/**
	 * set general settings
	 */
	public static function setGeneralSettings(UniteCreatorSettings $objSettings){
		
		//add stand alone settings
		
		if(UniteProviderFunctionsUC::$isStandAlone == true){
			
			$filepathGeneralStandAlone = GlobalsUC::$pathProvider."settings/general_settings_standalone.xml";
			UniteFunctionsUC::validateFilepath($filepathGeneralStandAlone, "Provider general settings stand alone");
			$objSettings->addFromXmlFile($filepathGeneralStandAlone);
			
		}
		
		//add general settings
		$filepathGeneral = GlobalsUC::$pathProvider."settings/general_settings.xml";
		UniteFunctionsUC::validateFilepath($filepathGeneral, "Provider general settings");
		$objSettings->addFromXmlFile($filepathGeneral);
		
		$objSettings = self::modifyGeneralSettingsProvider($objSettings);
		
		
		return($objSettings);
	}
	
	
	/**
	 * runs right after the load of all files
	 */
	public static function globalInit(){
		
		self::add_filter(UniteCreatorFilters::FILTER_MODIFY_GENERAL_SETTINGS, array("HelperProviderUC", "setGeneralSettings"));
		
	}
	
	private function a____________TOOLBAR____________(){}
	
	
	/**
	 * preview addon
	 */
	public static function addToolbar_layoutsLink(){
	
		$urlLayouts = HelperUC::getViewUrl_LayoutsList();
	
		$textLayouts = HelperUC::getText("my_layouts");
	
		$html = "";
		$html .= '<a href="'.$urlLayouts.'" class="unite-button-secondary " >'.$textLayouts.'</a>';
		$html .= "<span class='uc-toolbar-sap'></span>";
		
		$bar = JToolbar::getInstance('toolbar');
		$bar->appendButton('Custom', $html, "link_layouts");
		
		return($html);
	}
	
	
	/**
	 * add library link
	 */
	public static function addToolbar_libraryLink($position = "right"){
		
		$urlLibrary = HelperUC::getViewUrl("library");
	
		$textLibrary = __("My Library", BLOXBUILDER_TEXTDOMAIN);
	
		$html = "";
		$html .= '<a href="'.$urlLibrary.'" class="unite-button-secondary " >'.$textLibrary.'</a>';
		$html .= "<span class='uc-toolbar-sap'></span>";
		
		if($position == "right")
			$class = "link_align_right";
		else
			$class = "link_align_left";
		
		
		$bar = JToolbar::getInstance('toolbar');
		$bar->appendButton('Custom', $html, $class);
	}
	
	
	/**
	 * preview addon
	 */
	public static function addToolbar_addonsLink(){
	
		$urlAddons = HelperUC::getViewUrl_Addons();
		
		$textAddons = __("My Addons", BLOXBUILDER_TEXTDOMAIN);
		
		$html = "";
		$html .= '<a href="'.$urlAddons.'" class="unite-button-secondary " >'.$textAddons.'</a>';
		$html .= "<span class='uc-toolbar-sap'></span>";
		
		$bar = JToolbar::getInstance('toolbar');
		$bar->appendButton('Custom', $html, "link_addons");
		
		return($html);
	}
	
	/**
	 * preview addon
	 */
	public static function addToolbar_templatesLink($alignRight = true){
		
		$urlTemplates = HelperUC::getViewUrl_TemplatesList();
		
		$textTemplates = __("My Templates", BLOXBUILDER_TEXTDOMAIN);
		
		$html = "";
		$html .= '<a href="'.$urlTemplates.'" class="unite-button-secondary " >'.$textTemplates.'</a>';
		$html .= '<span class="uc-toolbar-sap"></span>';
		
		$class = "link_align_right";
		if($alignRight == false)
			$class = "link_align_left";
		
			
		$bar = JToolbar::getInstance('toolbar');
		$bar->appendButton('Custom', $html, $class);
				
		return($html);
	}
	
	
}