<?php
/**
 * @package Blox Page Builder
 * @author UniteCMS.net
 * @copyright (C) 2017 Unite CMS, All Rights Reserved. 
 * @license GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * */
defined('_JEXEC') or die('Restricted access');

	class UniteFunctionJoomlaUC{
		
		const TABLE_MODULES = "#__modules";
		const TABLE_EXTENSIONS = "#__extensions";
		const TABLE_CATEGORIES = "#__categories";
		const TABLE_CONTENT = "#__content";
		const TABLE_MENUTYPES = "#__menu_types";
		const TABLE_MENU = "#__menu";
		
		
		private static $arrControls;
		public static $componentName;	//current component name. have to be set on include.
		public static $app;
		private static $db;
		
		
		/**
		 * get db
		 */
		private static function getDB(){
			
			if(empty(self::$db))
				self::$db = new UniteCreatorDB();
			
			return(self::$db);
		}
		
		
		/**
		 * get menus array
		 */
		public static function getArrMenus(){
			
			$db = self::getDB();
			$response = $db->fetch(self::TABLE_MENUTYPES);
			
			$arrMenu = array();
			if(empty($response))
				return($arrMenu);
			
			foreach($response as $menu){
				$type = $menu["menutype"];
				$title = $menu["title"];
				$arrMenu[$type] = $title;
			}
			
			return($arrMenu);
		}
		
		
		/**
		 * get active menu item
		 */
		public static function getActiveMenuItemID(){
			
			$menu = JFactory::getApplication()->getMenu();
						
			if(empty($menu))
				return(null);
			
			$activeMenu = $menu->getActive();
			if(empty($activeMenu))
				return(null);
			
			$menuItemID = $activeMenu->id;
			
			return($menuItemID);
		}
		
		
		/**
		 * get menu items
		 */
		public static function getMenuItems($menuType, $getSubmenu = true){
			
			$activeItemID = self::getActiveMenuItemID();
			
			$db = self::getDB();
			 
			$arrItems = $db->fetch(self::TABLE_MENU,array("menutype"=>$menuType,"published"=>1));
									
			//collect items
			$arrSubItems = array();
			$arrItemsOutput = array();

			$currentParentID = null;
			
			foreach($arrItems as $key => $item){
				
				$params = UniteFunctionsUC::jsonDecode($item["params"]);
				
				$itemID = UniteFunctionsUC::getVal($item, "id");
				$isActive = false;
				if($itemID == $activeItemID)
					$isActive = true;
				
				$level = UniteFunctionsUC::getVal($item, "level");
				
				$type = UniteFunctionsUC::getVal($item, "type");
				
				//get link
				$link = UniteFunctionsUC::getVal($item, "link");
				
				
				switch($type){
					
					case 'separator':
					case 'heading':
					break;
					case 'url':
						if ((strpos($link, 'index.php?') === 0) && (strpos($link, 'Itemid=') === false)){
							$link = $link . '&Itemid=' . $itemID;
						}
					break;
					case 'alias':
						$aliasOption = UniteFunctionsUC::getVal($params, "aliasoptions");
						$link = 'index.php?Itemid=' .$aliasOption;
					break;
					default:
						$link = 'index.php?Itemid=' . $itemID;
					break;
				}
				
				if ((strpos($link, 'index.php?') !== false) && strcasecmp(substr($link, 0, 4), 'http')){
					$isSecure = UniteFunctionsUC::getVal($params, "secure");
										
					$link = JRoute::_($link, true, $isSecure);
										
				}
				else{
					$link = JRoute::_($link);
				}
				
				$link = htmlspecialchars($link, ENT_COMPAT, 'UTF-8', false);
				
				
				$arrItem = array();
				$arrItem["id"] = $itemID;
				$arrItem["title"] = UniteFunctionsUC::getVal($item, "title");
				$arrItem["alias"] = UniteFunctionsUC::getVal($item, "alias");
				$arrItem["link"] = $link;
				$arrItem["active"] = $isActive;
				$arrItem["submenu"] = array();
				$arrItem["current"] = false;
				
				//add level 1 menu
				if($level == 1){
					$arrItemsOutput["item_".$itemID] = $arrItem;
					continue;
				}

				if($level == 2){
					$parentID = UniteFunctionsUC::getVal($item, "parent_id");
					
					if($isActive == true)
						$currentParentID = $parentID;
					
					$submenuKey = "item_{$parentID}";
					
					if(!isset($arrSubItems[$submenuKey]))
						$arrSubItems[$submenuKey] = array();
					
					$arrSubItems[$submenuKey]["item_".$itemID] = $arrItem;
				}
				
				//skip third level meanwhile
			}
			
			//set current attribute
			if(!empty($currentParentID) && isset($arrItemsOutput["item_".$currentParentID]))
				$arrItemsOutput["item_".$currentParentID]["current"] = true;
			
			//collect submenus
			foreach($arrSubItems as $parentKey=>$submenu){
				
				if(!isset($arrItemsOutput[$parentKey]))
					continue;
				
				$arrItemsOutput[$parentKey]["submenu"] = $submenu;
				unset($arrSubItems[$parentKey]);
			}

			
			return($arrItemsOutput);
		}
		
		private function __________MENUS___________(){}		
		
		/**
		 * check if k2 extension exists
		 */
		public static function isK2Exists(){
			$db = self::getDB();
			$response = $db->fetch(self::TABLE_EXTENSIONS, "name='com_k2' and enabled=1");
			
			$isExists = !empty($response);
			
			return($isExists);
		}
		
		/**
		 * get k2 content by id
		 */
		public static function getK2ItemContent($id){
		    
		    if(empty($id))
		        return("");
		    
		    try{
		        
		        $k2Item = new UniteJoomlaK2ItemUC();
		        $k2Item->initByID($itemID);
		        $content = $k2Item->getContent();
		        
		        return($content);
		        
		    }catch(Exception $e){
		        
		        $message = $e->getMessage();
		        return($message);
		    }
		    
		    return ("");		    
		}
		
		
		/**
		 * get module record by ID
		 */
		public static function getModuleByID($id){
			$id = (int)$id;
			$db = self::getDB();
			
			$record = $db->fetchSingle(self::TABLE_MODULES,"id=$id");
			
			return($record);
		}
		
		
		/**
		 * get array of modules for front end
		 */
		public static function getArrFrontModulesShort(){
			
			$db = self::getDB();
			
			$where = "published = 1 and access = 1";
			
			$records = $db->fetch(self::TABLE_MODULES, $where);
			
			$arrModules = array();
			
			foreach($records as $record){
				
				$id = UniteFunctionsUC::getVal($record, "id");
				$title = UniteFunctionsUC::getVal($record, "title");
				$module = UniteFunctionsUC::getVal($record, "module");
				
				$showTitle = "$title ($module)";
				
				$arrModules[$id] = $showTitle;
			}
			
			return($arrModules);
		}
		
		
		/**
		 * add toolbar custom button
		 */
		public static function addToolbarCustomButton($html, $name){
			
			$bar = JToolbar::getInstance('toolbar');
			$bar->appendButton('Custom', $html, $name);
		}
		
		
		/**
		 * 
		 * return if current joomla version is joomla 3
		 */
		public static function isJoomla3(){
			
			if(defined("JVERSION")){
				$version = JVERSION;
				$version = (int)$version;
				return($version == 3);
			}
			
			if(class_exists("JVersion")){
				$jversion = new JVersion;
				$version = $jversion->getShortVersion();
				$version = (int)$version;
				return($version == 3);
			}			
			
			return(!defined("DS"));
		}
		
		
		/**
		 * 
		 * set controls array
		 */
		public static function setControls($arrControls){
			self::$arrControls = $arrControls;
		}
		
		
		/**
		 * 
		 * put the label and the input of some form field
		 */
		public static function putFormField($form,$name,$group=null){
			echo $form->getLabel($name,$group);
			echo $form->getInput($name,$group);
		}
		
		/**
		 * 
		 * print fieldset box
		 */
		public static function putHtmlFieldsetBox($form,$name,$boxTitle="Settings"){

			if(empty($form))
				UniteFunctionsUG::throwError("Form not found!!!");
				
			?>
			
				<fieldset class="adminform unite-adminform">
					<legend><?php echo $boxTitle?></legend>
					<ul class="adminformlist unite-adminformlist">
					<?php
						$fieldset = $form->getFieldset($name);
						if(empty($fieldset))
							UniteFunctionsUG::throwError("fieldset with name: $name not found.");
						
						foreach($fieldset as $key=>$field){
							?>
								<li><?php echo $field->label; ?>
								<?php echo $field->input; ?></li>
							<?php 
						}
					?>
					</ul>
				</fieldset>
			<?php
		}
		
		/**
		 * 
		 * add script declaration wrapper
		 */
		public static function addScriptDeclaration($script){
			$document = JFactory::getDocument();
			$document->addScriptDeclaration($script);
		}
		
		/**
		 * 
		 * add script declaration wrapper
		 */
		public static function addHtmlToHead($html){
			$document = JFactory::getDocument();
			$document->addCustomTag($html);
		}
		
		
		/**
		 *
		 * add script declaration wrapper
		 */
		public static function addStyleDeclaration($style){
			$document = JFactory::getDocument();
			$document->addStyleDeclaration($style);
			
		}
		
		
		/**
		 * 
		 * add script wrapper
		 */
		public static function addScript($urlScript){
			$document = JFactory::getDocument();
			$document->addScript($urlScript);
		}

		/**
		 * 
		 * add style wrapper
		 */
		public static function addStyle($urlStyle,$id=null){
			$document = JFactory::getDocument();
			$attribs = array();
			if(!empty($id))
				$attribs["id"]=$id;
			
			$document->addStyleSheet($urlStyle,"text/css",null,$attribs);
		}
		
		/**
		 * 
		 * encode array to registry (json) for saving, in some array of items
		 * 
		 */		
		public static function encodeArrayToRegistry($arr,$field){
 
			if(!isset($arr[$field]))
				return("");
				
			if(!is_array($arr[$field]))
				return($arr[$field]);
			
			$registry = new JRegistry();
			$registry->loadArray($arr[$field]);
			$value = $registry->toString('JSON');
			
			return($value);
		}
		
		/**
		 * 
		 * decode some array item to registry
		 */
		public static function decodeRegistryToArray($arr,$field){
			
			$output = array();
			if(!isset($arr[$field]))
				return($output);
				
			$value = $arr[$field];	
			if(is_array($value))
				return($value);
			
			$registry = new JRegistry();
			$registry->loadString($value,'JSON');
			$output = $registry->toArray();
			
			return($output);
		}
		
		/**
		 * 
		 * get form field value by types.
		 */
		public static function getFormFieldValue($form,$field,$group=null){
			$objField = $form->getField($field,$group);
			
			$value = $objField->value;
			$type = strtolower($objField->type);
			
			switch($type){
				case "mycheckbox":						
					$value = $objField->isChecked();
				break;
			}
			
			return($value);
		}
		
		
		/**
		 * 
		 * hide some form field
		 * @param $form
		 */
		public static function hideFormField(JForm $form,$field, $group=""){
			$class = $form->getFieldAttribute($field, "class","",$group);
			if(!empty($class))
				$class .= " hidden";
			else
				$class == "hidden";
			
			$form->setFieldAttribute($field, "hidden", "true",$group);
			$form->setFieldAttribute($field, "class", $class,$group);
		}
		
		
		/**
		 * 
		 * set alias from title
		 */
		public static function normalizeAlias($alias){
			$alias = JFilterOutput::stringURLSafe($alias);
			
			if(trim(str_replace('-','',$alias)) == '')
				$alias = JFactory::getDate()->format("Y-m-d-H-i-s");
			
			return($alias);				
		}	
		
		/**
		 * 
		 * put multiple html option boxes
		 * exceptions - come delimited fieldset names. Those won't be shown
		 */
		public static function putHtmlFieldsetBoxes($form,$name,$exceptions=""){
			
			$arrExceptions = explode(",",$exceptions);
			
			$arrfieldsets = $form->getFieldsets($name);
		
			foreach($arrfieldsets as $name=>$arrFieldset){
				
				if(in_array($name, $arrExceptions))
					continue;
				
				self::putHtmlFieldsetBox($form,$arrFieldset->name,$arrFieldset->label);				
			}
		}
		
		
		/**
		 * check if url approved
		 * Enter description here ...
		 */
		private static function isUrlApproved($url, $arrWhite, $arrBlack){
			
			foreach($arrWhite as $urlWhite){
				if(strpos($url, $urlWhite) !== false)
					return(true);
			}
			
			foreach($arrBlack as $urlBlack){
				if(strpos($url, $urlBlack) !== false){
					return(false);
				}
			}
			
			
			return(true);
		}
		
		
		/**
		 * order css includes
		 */
		public static function orderCssIncludes(){
			$doc = JFactory::getDocument();
				
			if(!method_exists($doc,'getHeadData')){ 
				return;
			}
			
			//	looking for scripts
			$headers = $doc->getHeadData();
			
			dmp($headers);exit();
		}
		
		/**
		 * 
		 * disable mootools include
		 */
		public static function disableMootools(){
			
			JHtml::_('behavior.framework', true);
			JHtml::_('behavior.modal');
			
			//return(true);
			
			$doc = JFactory::getDocument();
				
			if(!method_exists($doc,'getHeadData')){ 
				return;
			}
			
			//	looking for scripts
			$headers = $doc->getHeadData();
						
			$scripts = isset($headers['scripts']) ? $headers['scripts'] : array();
			$headers['scripts'] = array();
			
			
			$arrBlackList = array(
				"mootools",
				"js/modal.js",
				"js/modal-uncompressed.js",
				"jquery-migrate.js",
				"helix",
				"media/jui/js/jquery.ui.core.min.js",
				"media/jui/js/jquery.ui.sortable.min.js",
				"js/caption.js",
				"media/jui/js/jquery-noconflict.js",
				"/joomla/media/jui/js/jquery-migrate.min.js"
			);
			
			
			$arrWhiteList = array(
				"jquery.min.js"
			);
			
			foreach($scripts as $url=>$type){
				
				$isApproved = self::isUrlApproved($url, $arrWhiteList, $arrBlackList);
				
				if($isApproved == false)
					continue;
				
				$headers['scripts'][$url] = $type;
				
			}
			
			
			$doc->setHeadData($headers); 	

			//self::orderCssIncludes();
		}


		/**
		 * disable joomla preview files
		 */
		public static function disableJoomlaPreviewFiles(){
		
			$doc = JFactory::getDocument();
			
			if(!method_exists($doc,'getHeadData')){ 
				return;
			}
			
			
			//	looking for scripts
			$headers = $doc->getHeadData();
						
			$scripts = isset($headers['scripts']) ? $headers['scripts'] : array();
			$headers['scripts'] = array();
			
			
			$arrBlackList = array(
				"mootools",
				"media/jui/js/jquery.ui.sortable.min.js",
				"media/jui/js/jquery.ui.core.min.js",
				"js/caption.js",
				"media/jui/js/jquery-noconflict.js",
				"/joomla/media/jui/js/jquery-migrate.min.js",
				"js/modal.js",
				"helix"
			);
			
			
			$arrWhiteList = array(
				"jquery.min.js"
			);
			
			foreach($scripts as $url=>$type){
				
				$isApproved = self::isUrlApproved($url, $arrWhiteList, $arrBlackList);
				
				if($isApproved == false)
					continue;
				
				$headers['scripts'][$url] = $type;
				
			}

			
			//dmp($headers);exit();
			
			$doc->setHeadData($headers);
			
		}
		
		
		/**
		 * 
		 * give joomla order to hide main menu. 
		 * this must be used on view.html.php
		 */
		public static function hideMainMenu(){
			JFactory::getApplication()->input->set('hidemainmenu', 1);
		}
		
		/**
		 * 
		 * get component url - site side
		 * 
		 */
		public static function getUrlComponent($args,$component=""){
			if(empty($component))
				$component = self::$componentName;
			$url = juri::root()."index.php?option=".$component."&".$args;
			
			return($url);
		}
		
		
		/**
		 * 
		 * get view url (admin side)
		 */
		public static function getViewUrl($view,$layout="default",$args="",$component=""){
			
			if(empty($component))
				$component = self::$componentName;
				
			$url = "index.php?option=".$component;
			$url .= "&view=$view";
			
			//add layout
			if(!empty($layout))
				$url .= "&layout=".$layout; 
			
			//add additional arguments
			if(!empty($args))
				$url .= "&".$args;
			
			//$url = JURI::root().$url;
			
			$url = JRoute::_($url,false);
			
			return($url);
		}
		
		/**
		 * Get url of image for output
		 */
		public static function getImageOutputUrl($filename,$width=0,$height=0,$exact=false,$encode=true){
			
			//exact validation:
			if(($exact == "true" || $exact == true) && (empty($width) || empty($height) ))
				UniteFunctionsUG::throwError("Exact must have both - width and height");
						
			if($encode == true)
				$filename = base64_encode($filename);
			
			$url = "index.php?option=".self::$componentName."&task=showimage&img=$filename";
			
			if(!empty($width))
				$url .= "&w=".$width;
		
			if(!empty($height))
				$url .= "&h=".$height;
		
			if($exact == true)
				$url .= "&t=exact";
		
			if($encode == false)
				$url .= "&noencode=true";
			
			return($url);
		}
		
		
		/**
		 * 
		 * get image url from filename
		 */
		public static function getImageUrl($filename){
			$filenameLower = strtolower($filename);
			
			if(strpos($filenameLower, "http://") !== false)
				return($filename);
				
			if(strpos($filenameLower, "https://") !== false)
				return($filename);
				
			if(strpos($filenameLower, "www.") === 0)
				return($filename);
			
			$urlImage = JURI::root().$filename;
			
			if(JURI::getInstance()->isSSL() == true)
				$urlImage = str_replace("http://","https://",$urlImage);
			
			return($urlImage);
		}
		
		
		/**
		 * check if ssl or not
		 */
		public static function isSSL(){
			
			$isSSL = JURI::getInstance()->isSSL();
			
			return($isSSL);
		}
		
		
		/**
		 * 
		 * get image filepath from filename
		 * @param $filename
		 */
		public static function getImageFilepath($filename){
			$pathImage = JPATH_SITE."/".$filename;
			return($pathImage);
		}
		
		/**
		 * 
		 * get image filepath from image url (images folder)
		 */
		public static function getImageFilepathFromUrl($urlImage){
			$urlBase = GlobalsUG::$url_base;
			$pathImage = str_replace($urlBase, "", $urlImage);
			
			$filepathImage = GlobalsUG::$path_base.$pathImage;
			$filepathImage = realpath($filepathImage);
			
			return($filepathImage);
		}
		
		
		/**
		 * get image path from url
		 */
		public static function getPathImageFromUrl($urlImage){
			$urlBase = GlobalsUG::$url_base;
			$pathImage = str_replace($urlBase, "", $urlImage);
			
			return($pathImage);
		}
		
		/**
		 * get cache path. if not exists - try to crate it
		 */
		private static function getPathCache(){
			
			//set cache path
			$component = self::$componentName;
			
			$pathCache = JPATH_SITE."/cache/".$component."/";
			if(is_dir($pathCache))
				return($pathCache);
			
			@mkdir($pathCache);
			
			if(is_dir($pathCache))
				return($pathCache);
			
			//make media cache path
			$pathCache = JPATH_SITE."/media/".$component."/cache/";
			if(is_dir($pathCache))
				return($pathCache);
			
			@mkdir($pathCache);
			
			if(is_dir($pathCache))
				return($pathCache);
			
			//make component cache path
			$pathCache = JPATH_COMPONENT_SITE."/cache/";
			
			return($pathCache);
		}
		
		

		/**
		 * 
		 * get post or get application
		 */
		public static function getPostGetVar($name,$default="",$filter="STRING"){
			if(empty(self::$app))
				self::$app = JFactory::getApplication();
			
			$jinput = self::$app->input;
			$var = $jinput->get($name,$default,$filter);
			return($var);
		}
		
		
		/**
		 * 
		 * clear params array from client side
		 */
		public static function clearParamsArray($arrParams){
			$arrNew = array();
			foreach($arrParams as $key=>$value){
				$key = str_replace("jform_params_", "", $key);
				$key = str_replace("jform_", "", $key);
				$arrNew[$key] = $value;
			}
			return($arrNew);
		}
		
		/**
		 * 
		 * check if jquery script included
		 */
		public static function isJqueryIncluded(){
			$document = JFactory::getDocument();
			$arrScripts = $document->_scripts;
						
			foreach($arrScripts as $filepath=>$arrDetails){
				$filename = basename($filepath);
				$filename = strtolower($filename);
				$pattern = '#jquery([0-9\.-]|core|min|pack)*?.js#';

				preg_match_all($pattern, $filename, $matches);
				if(is_array($matches) && !empty($matches))
					if(!empty($matches[0]))
						return(true);
			}
			return(false);
		}
		
		
		/**
		 * add jquery include
		 */
		public static function addjQueryInclude(){
			
			JHtml::_('jquery.framework');
			
			/*
			if(self::isJqueryIncluded() == true)
				return(false);
			
			$document = JFactory::getDocument();
			
			$jsPrefix = "http";
			if(JURI::getInstance()->isSSL() == true)
				$jsPrefix = "https";
			
			if(!empty($urljQuery))
				$document->addScript($urljQuery."?app={$app}");
			else
				$document->addScript("{$jsPrefix}://ajax.googleapis.com/ajax/libs/jquery/1.10.2/jquery.min.js?app={$app}");
			*/
			
		}
		
		
		/**
		 * get joomla categories array
		 */
		public static function getArrCategoriesShort(){
			
			$db = self::getDB();
			
			$arrCats = $db->fetch(self::TABLE_CATEGORIES,"published=1 and extension='com_content'","title");
			
			$output = array();
			$output["[Not Selected]"] = "";
			
			foreach($arrCats as $cat){
				$id = $cat["id"];
				$title = $cat["title"];
				$alias = $cat["alias"];
				
				if(isset($output[$title]))
					$title = $title." ($alias)";
				
				$output[$title] = $id;
			}
			
			return($output);
		}
		
		
		/**
		 * get orderby
		 */
		public static function getArticlesOrderby($addNoSelect = false){
			
			$arrOrderby = array();
			
			if($addNoSelect == true)
				$arrOrderby["[Not Selected]"] = "";
			
			$arrOrderby["Ordering"] = "ordering";
			$arrOrderby["Title"] = "title";
			$arrOrderby["Recently Added"] = "created";
			$arrOrderby["Last Published"] = "publish_up";
			$arrOrderby["Last Modified"] = "modified";
			$arrOrderby["Title"] = "title";
			$arrOrderby["Hits"] = "hits";
			$arrOrderby["Random"] = "random";
			
			return($arrOrderby);
		}
		
		
		/**
		 * get orderby
		 */
		private static function getArticlesRecords_orderby($orderBy, $dir){
			
			if(empty($orderBy))
				return("");
			
			$strOrder = $orderBy;
			
			if($dir == "desc")
				$strOrder .= " ".$dir;
				
			return($strOrder);
		}
		
		
		/**
		 * get articles list by category and other params
		 */
		public static function getArticlesRecords($filters){
			
			
			$db = self::getDB();
			
			$category = UniteFunctionsUC::getVal($filters, "category");
			$category = (int)$category;
						
			$arrWhere = array();
			$arrWhere["state"] = 1;
			
			if(!empty($category))
				$arrWhere["catid"] = $category;

			//--- set order--- 
			
			$strOrder = "";
			
			$orderby1 = UniteFunctionsUC::getVal($filters, "orderby");
			if($orderby1 == "random"){
				$strOrder = "RAND()";
			}else{
				$orderdir1 = UniteFunctionsUC::getVal($filters, "orderdir");
				$strOrder = self::getArticlesRecords_orderby($orderby1, $orderdir1);
				
				$orderby2 = UniteFunctionsUC::getVal($filters, "orderby2");
				$orderdir2 = UniteFunctionsUC::getVal($filters, "orderdir2");
				$strOrder2 = self::getArticlesRecords_orderby($orderby2, $orderdir2);
				
				//combine 2 orders
				if(!empty($strOrder2)){
					if(!empty($strOrder))
						$strOrder .= ", ";
					$strOrder .= $strOrder2;
				}
			}
				
			$addSql = "";
			
			// ---- set limit ----
			
			$limit = UniteFunctionsUC::getVal($filters, "limit");
			if(!empty($limit))
				$addSql .= "limit {$limit}";
						
			$records = $db->fetch(self::TABLE_CONTENT, $arrWhere, $strOrder, "", $addSql);
			
			return($records);
		}
		
	}
	

?>