<?php
/**
 * @package Blox Page Builder
 * @author UniteCMS.net
 * @copyright (C) 2017 Unite CMS, All Rights Reserved. 
 * @license GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * */
defined('_JEXEC') or die('Restricted access');

class UniteJoomlaArticleUC extends UniteElementsBaseUC{
	
	const TABLE_CONTENT = "#__content";
	
	private $id;
	private $record;
	private $data;
	
	
	/**
	 * validate that the article is inited
	 */
	private function validateInited(){
		if(empty($this->id))
			UniteFunctionsUC::throwError("The article is not inited");
	}
	
	
	/**
	 * get article title
	 */
	public function getTitle($specialChars = false){
		$this->validateInited();
		
		$title = UniteFunctionsUC::getVal($this->record, "title");
		if($specialChars == true)
			$title = htmlspecialchars($title);
			
		return($title);
	}
	
	/**
	 * get front end link to the article
	 */
	public function getLink(){
		
		$this->validateInited();
			
		$com_path = JPATH_SITE . '/components/com_content/';
		JLoader::register('ContentHelperRoute', $com_path . 'helpers/route.php');
		
		if(class_exists("ContentHelperRoute"))
			$link = GlobalsUC::$url_base.ContentHelperRoute::getArticleRoute($this->record['id'], $this->record['catid'], $this->record['language']);
		else
			$link = GlobalsUC::$url_base."index.php?option=com_content&view=article&id=".$this->id;
		
		$link = htmlspecialchars_decode($link);
		
		return($link);
	}
	
	
	/**
	 * get date
	 */
	private function getDateStamp(){
		
		$date = UniteFunctionsUC::getVal($this->data, "published_up");
		if(empty($date))
			$date = UniteFunctionsUC::getVal($this->data, "created");
		
		$stamp = strtotime($date);
		
		return($stamp);
	}
	
	
	/**
	 * get article data
	 */
	public function getData(){
		
		if(!empty($this->data))
			return($this->data);
		
		$this->validateInited();
		
		$this->data = $this->record;
		
		//decode json
		$arrFieldsDecode = array("images","attribs","urls","metadata");
		
		foreach($arrFieldsDecode as $key){
			$content = UniteFunctionsUC::getVal($this->data, $key);
			$this->data[$key] = UniteFunctionsUC::jsonDecode($content);
		}
		
		$this->data["link"] = $this->getLink();
		$this->data["datestamp"] = $this->getDateStamp();
		
		return($this->data);
	}
	
	
	/**
	 * get intro image url
	 */
	public function getUrlImageIntro(){
		
		$data = $this->getData();
		
		$images = UniteFunctionsUC::getVal($data, "images");
		$urlIntro = UniteFunctionsUC::getVal($images, "image_intro");
		
		return($urlIntro);
	}
	
	
	/**
	 * init article by db record
	 */
	public function initByRecord($record){
		
		$this->record = $record;
		$this->id = $record["id"];
	}
	
	
	/**
	 * init by id
	 */
	public function initByID($id){
		
		$id = (int)$id;
		$record = $this->db->fetchSingle(self::TABLE_CONTENT,"id=".$id);
		
		$this->initByRecord($record);
	}
	
}

