<?php
/**
 * @package Blox Page Builder
 * @author UniteCMS.net
 * @copyright (C) 2017 Unite CMS, All Rights Reserved. 
 * @license GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * */
defined('_JEXEC') or die('Restricted access');

class UniteSettingsOutputUC extends UniteSettingsOutputUCWork{
	
	
	
	/**
	 * draw post picker input
	 * value is post id
	 */
	protected function drawPostPickerInput($setting, $postType = "article"){
			
			$value = UniteFunctionsUC::getVal($setting, "value");
			
			$postTitle = "";
			
			if(empty($value))
				$value = "";
			else{		//get post title
				
				try{
					$postTitle = UniteProviderFunctionsUC::getPostTitleByID($value);
					
				}catch(Exception $e){
					$value = "";
				}
			}
			
			$class = $this->getInputClassAttr($setting, "", "unite-postpicker-input");
			$addHtml = $this->getDefaultAddHtml($setting);
			$addHtml .= " data-initval-title=\"{$postTitle}\"";
			
			$dialogTitle = __("Select Article", BLOXBUILDER_TEXTDOMAIN);
			$textButton = __("Select Article", BLOXBUILDER_TEXTDOMAIN);
			
			if($postType == "k2"){
				$dialogTitle = __("Select K2 Item", BLOXBUILDER_TEXTDOMAIN);
				$textButton = __("Select K2 Item", BLOXBUILDER_TEXTDOMAIN);
			}
			
			?>
			<div class="unite-settings-postpicker">
				<input type="text" disabled id="<?php echo $setting["id"]?>"  data-postid="<?php echo $value?>" name="<?php echo $setting["name"]?>" <?php echo $class?> value="<?php echo $postTitle?>" <?php echo $addHtml?> />
				<a class="unite-button-secondary uc-button-select-post" data-posttype="<?php echo $postType?>" data-dialogtitle="<?php echo $dialogTitle?>"><?php echo $textButton?></a>
			</div>
			
		<?php
	}


	/**
	 * draw module picker input
	 */
	protected function drawModulePickerInput($setting){
		
		$arrModules = UniteFunctionJoomlaUC::getArrFrontModulesShort();
		
		$textNotSelected = __("[Choose Module]", BLOXBUILDER_TEXTDOMAIN);
		
		$arrItems = array();
		$arrItems[$textNotSelected] = "";
		
		foreach($arrModules as $moduleID=>$moduleText)
			$arrItems[$moduleText] = $moduleID;
		
		$setting["items"] = $arrItems;
		
		$this->drawSelectInput($setting);
		
	}
	
	
	/**
	 * draw k2 picker input
	 */
	protected function drawK2ItemPickerInput($setting){
		
		$isK2Exists = UniteFunctionJoomlaUC::isK2Exists();
		if($isK2Exists == false)
			echo "The K2 Component not exists or disabled. Please install and turn it on and this setting will work.";
		else
			$this->drawPostPickerInput($setting, "k2");
		
	}
	
	/**
	 * get editor buttons array
	 */
	private function getArrEditorButtons(){
		
		$arrButtons = array();
		
		$button = new JObject();
		$button->class = "unite-button-secondary";
		$button->name = "uc-button-add-image";
		$button->text = __("Add Image",BLOXBUILDER_TEXTDOMAIN);
		$button->link = "javascript:void(0)";
		
		$arrButtons[] = $button;
		
		return($arrButtons);
	}
	
	
	/**
	 * draw editor input in sidebar
	 */
	private function drawEditorInputSidebar($setting){
		
		$name = $setting["name"];
		$id = $setting["id"];
		$idFullEditor = $id."_full";
		$idDialog = $id."_dialog";
		
		$title = $setting["text"];
		$dialogTitle = __("Edit", BLOXBUILDER_TEXTDOMAIN)." ".$title;
		
		$addHtml = $this->getDefaultAddHtml($setting);
		
		$value = UniteFunctionsUC::getVal($setting, "value");
		$value = htmlspecialchars($value, ENT_COMPAT, 'UTF-8');
		
		?>
		<div class='unite-editor-setting-wrapper' <?php echo $addHtml?>>
			
			<div id="<?php echo $id?>" data-name='<?php echo $name?>' class='unite-setting-inline-editor'>
				<?php echo $value?>
			</div>
			
		</div>
		<div id="<?php echo $idDialog?>" class="unite-settings-editor-inline-full-dialog unite-inputs" style="display:none" title="<?php echo $dialogTitle?>">
			<textarea id="<?php echo $idFullEditor?>" class="unite-settings-textarea-wide"></textarea>
		</div>
		<?php
		
	}
	
	
	/**
	 * draw editor input
	 */
	protected function drawEditorInput($setting){
		
		if($this->isSidebar == true){
			$this->drawEditorInputSidebar($setting);
			return(false);
		}
		
		
		$disabled = "";
		if (isset($setting["disabled"]))
			$disabled = 'disabled="disabled"';
		
		$style = "";
		if(isset($setting["style"]))
			$style = "style='".$setting["style"]."'";
		
		$editor = JEditor::getInstance("tinymce");
		
		$name = $setting["name"];
		$value = UniteFunctionsUC::getVal($setting, "value");
		$value = htmlspecialchars($value, ENT_COMPAT, 'UTF-8');
		$width = "100%";
		$height = "200";
		$cols = 5;
		$rows = 3;
		
		//exclude buttons, leave only article and image if exists
		$buttons = array("pagebreak", "readmore");
		$id = $setting["id"];
		
		$addHtml = $this->getDefaultAddHtml($setting);
		
		$params = array();
		$params["html_height"] = $height;
		$params["mode"] = 1;
		
		$html = "<div class='unite-editor-setting-wrapper unite-editor-joomla' {$addHtml} >";
		
		$html .= $editor->display(
				$name, $value , $width, $height, $cols, $rows,
				$buttons, $id, null,
				null, $params);
		
		$html .= "</div>";
		
		echo $html;
	}
	
	
	/**
	 * draw setting input by type
	 */
	protected function drawInputs($setting){
				
		switch($setting["type"]){
			
			case UniteCreatorSettings::TYPE_MODULE:
				$this->drawModulePickerInput($setting);
			break;
			case UniteCreatorSettings::TYPE_JOOMLA_K2_ITEM:
				$this->drawK2ItemPickerInput($setting);
			break;
			default:
				parent::drawInputs($setting);
			break;
		}
		
	}	
	
	
}